%% This script is testing the functionalities of the Dipolar Gas Simulator
%
%  Important:  Run only sectionwise!! 

%% - Create Simulator, Potential and Calculator object with specified options

OptionsStruct = struct;

OptionsStruct.NumberOfAtoms           = 1E6;
OptionsStruct.DipolarPolarAngle       = pi/2;
OptionsStruct.DipolarAzimuthAngle     = 0;
OptionsStruct.ScatteringLength        = 89;

OptionsStruct.TrapFrequencies         = [50, 10, 150];
OptionsStruct.TrapDepth               = 5;
OptionsStruct.BoxSize                 = 15;
OptionsStruct.TrapPotentialType       = 'Harmonic';

OptionsStruct.NumberOfGridPoints      = [64, 32, 64];
OptionsStruct.Dimensions              = [20, 40, 80];
OptionsStruct.CutoffType              = 'Cylindrical';
OptionsStruct.SimulationMode          = 'ImaginaryTimeEvolution'; % 'ImaginaryTimeEvolution' | 'RealTimeEvolution'
OptionsStruct.TimeStepSize            = 50E-6;                    % in s
OptionsStruct.NumberOfTimeSteps       = 200;                      % in s
OptionsStruct.EnergyTolerance         = 5E-10;

OptionsStruct.JobNumber               = 1;
OptionsStruct.SaveData                = true;
OptionsStruct.SaveDirectory           = './Data';
options                               = Helper.convertstruct2cell(OptionsStruct);
clear OptionsStruct

sim            = Simulator.DipolarGas(options{:});
pot            = Simulator.Potentials(options{:});
sim.Potential  = pot.trap(); % + pot.repulsive_chopstick();

%-% Run Simulation %-%
[Params, Transf, psi, V, VDk] = sim.run();

%% - Plot numerical grid
Plotter.visualizeSpace(Transf)
%% - Plot trap potential
Plotter.visualizeTrapPotential(V,Params,Transf)
%% - Plot initial wavefunction
Plotter.visualizeWavefunction(psi,Params,Transf)