function [Params] = setupParameters(this)
    CONSTANTS      = Helper.PhysicsConstants;
    hbar           = CONSTANTS.PlanckConstantReduced; % [J.s]
    w0             = 2*pi*100;                        % Angular frequency unit [s^-1]
    
    % Mass, length scale
    Params.m       = CONSTANTS.Dy164Mass;
    l0             = sqrt(hbar/(Params.m*w0)); % Defining a harmonic oscillator length

    % Number of points in each direction
    Params.Nx      = this.NumberOfGridPoints(1);
    Params.Ny      = this.NumberOfGridPoints(2);
    Params.Nz      = this.NumberOfGridPoints(3);
    
    % Dimensions (in units of l0)
    Params.Lx      = this.Dimensions(1);
    Params.Ly      = this.Dimensions(2);
    Params.Lz      = this.Dimensions(3);
    
    % Trapping frequencies
    Params.wx      = 2*pi*this.TrapFrequencies(1);
    Params.wy      = 2*pi*this.TrapFrequencies(2);
    Params.wz      = 2*pi*this.TrapFrequencies(3);
    
    % Trap depth and box size for box potentials
    Params.boxdepth = this.TrapDepth/(Params.wz/w0);  % The depth of the box
    Params.boxsize  = this.BoxSize;                % The size of the box

    % ================ Parameters defined by those above ================ %
    
    % Trap gamma
    Params.gx      = (Params.wx/w0)^2;
    Params.gy      = (Params.wy/w0)^2;
    Params.gz      = (Params.wz/w0)^2;

end